
#include <zephyr/net/mqtt.h>
#include <zephyr/net/socket.h>
#include <zephyr/random/random.h>

#include <zephyr/logging/log.h>
LOG_MODULE_REGISTER(mqtt);


#define SERVER_PORT          1883
#define SERVER_ADDR          "91.121.93.94"
#define MQTT_CLIENTID		 "zephyr_publisher"
#define APP_MQTT_BUFFER_SIZE 128

struct mqtt_client mqtt_client;
struct sockaddr_storage broker;
bool connected = false;
uint8_t rx_buffer[APP_MQTT_BUFFER_SIZE];
uint8_t tx_buffer[APP_MQTT_BUFFER_SIZE];
static int nfds;


static void clear_fds(void)
{
	nfds = 0;
}

static char *get_mqtt_payload(enum mqtt_qos qos)
{
	static char payload[] = "DOORS:OPEN_QoSx";

	payload[strlen(payload) - 1] = '0' + qos;

	return payload;
}

void mqtt_evt_handler(struct mqtt_client *const client, const struct mqtt_evt *evt)
{
	int err;

	switch (evt->type)
    {
        case MQTT_EVT_CONNACK:
            if (evt->result != 0)
            {
                LOG_ERR("MQTT connect failed %d", evt->result);
                break;
            }

            connected = true;
            LOG_INF("MQTT client connected!");

            break;

        case MQTT_EVT_DISCONNECT:
            LOG_INF("MQTT client disconnected %d", evt->result);

            connected = false;
            clear_fds();

            break;

        case MQTT_EVT_PUBACK:
            if (evt->result != 0)
            {
                LOG_ERR("MQTT PUBACK error %d", evt->result);
                break;
            }

            LOG_INF("PUBACK packet id: %u", evt->param.puback.message_id);

            break;

        case MQTT_EVT_PUBREC:
            if (evt->result != 0)
            {
                LOG_ERR("MQTT PUBREC error %d", evt->result);
                break;
            }

            LOG_INF("PUBREC packet id: %u", evt->param.pubrec.message_id);

            const struct mqtt_pubrel_param rel_param = {
                .message_id = evt->param.pubrec.message_id
            };

            err = mqtt_publish_qos2_release(client, &rel_param);
            if (err != 0)
            {
                LOG_ERR("Failed to send MQTT PUBREL: %d", err);
            }

            break;

        case MQTT_EVT_PUBCOMP:
            if (evt->result != 0)
            {
                LOG_ERR("MQTT PUBCOMP error %d", evt->result);
                break;
            }

            LOG_INF("PUBCOMP packet id: %u", evt->param.pubcomp.message_id);

            break;

        case MQTT_EVT_PINGRESP:
            LOG_INF("PINGRESP packet");
            break;

        default:
            break;
	}
}

static void broker_init(void)
{
	struct sockaddr_in *broker4 = (struct sockaddr_in *)&broker;

	broker4->sin_family = AF_INET;
    // Set the Broker PORT
	broker4->sin_port = htons(SERVER_PORT);//8883 = 0x22b3 -> converts this to 0xb322 and stores in the variable
	// Set the Broker Address
    zsock_inet_pton(AF_INET, SERVER_ADDR, &broker4->sin_addr);
}

void mqtt_init()
{
    int rc;
	mqtt_client_init(&mqtt_client);

	broker_init();

	/* MQTT client configuration */
	mqtt_client.broker = &broker;
	mqtt_client.evt_cb = mqtt_evt_handler;
	mqtt_client.client_id.utf8 = (uint8_t *)MQTT_CLIENTID;
	mqtt_client.client_id.size = strlen(MQTT_CLIENTID);

	struct mqtt_utf8 username, password;
	username.utf8 = "Your_Username";
	username.size = strlen("Your_Username") -1;
	password.utf8 = "Your_Password";
	password.size = strlen("Your_Password") -1;
    mqtt_client.user_name = &username;
    mqtt_client.password = &password;
	mqtt_client.protocol_version = MQTT_VERSION_3_1_1;

	/* MQTT buffers configuration */
	mqtt_client.rx_buf = rx_buffer;
	mqtt_client.rx_buf_size = sizeof(rx_buffer);
	mqtt_client.tx_buf = tx_buffer;
	mqtt_client.tx_buf_size = sizeof(tx_buffer);
    mqtt_client.transport.type = MQTT_TRANSPORT_NON_SECURE;

    rc = mqtt_connect(&mqtt_client);
    if (rc != 0)
    {
        LOG_ERR("mqtt_connect %d\n\r", rc);
    }
    else
    {
        LOG_INF("MQTT CONNECT SUCCESSFUL");
    }
}

void mqtt_send_publish()
{
    int rc;
	struct mqtt_publish_param param;    

	param.message.topic.qos = MQTT_QOS_0_AT_MOST_ONCE;
	param.message.topic.topic.utf8 = (uint8_t *)"sensors";
	param.message.topic.topic.size = strlen(param.message.topic.topic.utf8);
	param.message.payload.data = get_mqtt_payload(MQTT_QOS_0_AT_MOST_ONCE);
	param.message.payload.len = strlen(param.message.payload.data);
	param.message_id = sys_rand32_get();
	param.dup_flag = 0U;
	param.retain_flag = 0U;

	rc = mqtt_publish(&mqtt_client, &param);
    if (rc < 0)
    {
        LOG_ERR("Mqtt Publish Error %d", rc);
    }
    else
    {
        LOG_INF("Mqtt Publish Success");
    }
}